<?php

/**
 * @file
 * Provides field hooks for nlmfield module.
 */


/**
 * Implements hook_field_info().
 */
function nlmfield_field_info() {
  return array(
    'nlmfield_contributor' => array(
      'label' => 'NLM - Contributor',
      'description' => t('Stores information about a person (name, affiliation etc.)'),
      'default_widget' => 'nlmfield_text',
      'default_formatter' => 'nlmfield_xml',
    ),
  );
}

/**
 * Implementation of hook_field_settings().
 */
function nlmfield_field_settings($op, $field) {
  switch ($op) {
    case 'database columns':
      return array(
        'xml' => array(
          'type' => 'text',
          'size' => 'big',
          'not null' => FALSE,
        ),
        'contrib_type' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
        'name_string' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
        'name_style' => array(
          'type' => 'varchar',
          'length' => '64',
          'not null' => FALSE,
        ),
        'name_sur' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
        'name_given' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
        'name_sort' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
        'role' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
        'aff' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
        'email' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
        'address' => array(
          'type' => 'varchar',
          'length' => '255',
          'not null' => FALSE,
        ),
      );
      break;
  }
}

/**
 * Implementation of hook_field
 */
function nlmfield_field($op, &$node, $field, &$items, $teaser, $page) {
  switch ($op) {
    case 'presave':
      if ($field['type'] === 'nlmfield_contributor') {
        foreach ($items as $delta => $item) {
          nlmfield_compute_values($item, 'nlmfield_contributor', $field['widget']['nlmfield_namespace']);
          $items[$delta] = $item;
        }

      }
    break;

  }
}


/**
 * Implements hook_content_is_empty().
 */
function nlmfield_content_is_empty($item, $field) {
    if($field['widget']['type'] == 'nlmfield_text'){
      return (empty($item['name_string']) && empty($item['name_sur']) &&  empty($item['role']) &&  empty($item['aff']) && empty($item['email']));
    } else if($field['widget']['type'] == 'nlmfield_xml'){
      if(empty($item['xml'])){
        return TRUE;
      } else {
        return FALSE;
      }
    }
}

/**
 * Implements hook_field_widget_info().
 */
function nlmfield_widget_info() {
  $widgets = array();
  $widgets['nlmfield_xml'] = array(
    'label' => t('XML'),
    'field types' => array('nlmfield_contributor'),
    'multiple values' => CONTENT_HANDLE_CORE,
  );
  $widgets['nlmfield_text'] = array(
    'label' => t('TEXT'),
    'field types' => array('nlmfield_contributor'),
    'multiple values' => CONTENT_HANDLE_CORE,
  );
  return $widgets;
}

/**
 * Implementation of hook_widget().
 */
function nlmfield_widget(&$form, &$form_state, $field, $items, $delta = 0) {
  $element = array(
    '#type' => $field['widget']['type'],
    '#default_value' => isset($items[$delta]) ? $items[$delta] : '',
    '#title' => $field['widget']['label'],
    '#weight' => $field['widget']['weight'],
    '#description' => $field['widget']['description'],
    '#required' => $field['required'],
    '#field' => $field,
  );
  return $element;
}

/**
 * Implements hook_field_widget_settings().
 */
function nlmfield_widget_settings($op, $widget) {
  switch($op){
    case 'form':
      $settings = $widget['settings'];
      $form = array();
      $form['nlmfield_namespace'] = array(
        '#type' => 'textfield',
        '#title' => t('Namespace'),
        '#default_value' => isset($settings['nlmfiel_namespace']) ? $settings['nlmfiel_namespace'] : 'nlm',
      );
      return $form;
      break;
    case 'save':
      return array('nlmfield_namespace');
      break;
  }
}


function nlmfield_process($element, $edit, $form_state, $form){
  $field = $form['#field_info'][$element['#field_name']];
  $widget = $field['widget'];
  $delta = $element['#delta'];
  $xml = $element['#default_value']['xml'];
  switch ($widget['type']) {
    case 'nlmfield_xml':
      $element['xml'] = array(
        '#title' => 'XML',
        '#type' => 'textarea',
        '#default_value' => isset($element['#value']['xml']) ? $element['#value']['xml'] : '',
      );
    break;

    case 'nlmfield_text':

      $element['xml']['#type'] = 'hidden';
      $element['xml']['#delta'] = $delta;
      $element['xml']['#field_name'] = $element['#field_name'];
      $element['xml']['#element_validate'] = array('nlmfield_field_widget_text_to_xml');

      $element['surname'] = array(
        '#type' => 'textfield',
        '#title' => t('Last Name'),
        '#default_value' => nlmfield_process_xpath('//surname', $xml),
        '#description' => t("Your last, or family, name, e.g. 'Higgs-Boson'."),
      );
      $element['contributor_type'] = array(
        '#type' => 'textfield',
        '#title' => t('Contributor type'),
        '#default_value' => nlmfield_process_xpath('//@contrib-type', $xml),
        '#description' => t("Specifies contribution type, typical values are author, editor etc"),
      );
      $element['string_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Name String'),
        '#default_value' => nlmfield_process_xpath('//string-name', $xml),
      );
      $element['given_names'] = array(
        '#type' => 'textfield',
        '#title' => t('First name and middle name'),
        '#default_value' => nlmfield_process_xpath('//given-names', $xml),
        '#description' => t("First or given name, e.g. 'Peter'."),
      );
      $element['mail'] = array(
        '#type' => 'textfield',
        '#title' => t('Email'),
        '#default_value' => nlmfield_process_xpath('//email', $xml),
        '#description' => t("Your email address, e.g. higgs-boson@gmail.com"),
      );
      $element['role'] = array(
        '#type' => 'textfield',
        '#title' => t('Occupation'),
        '#default_value' => nlmfield_process_xpath('//role', $xml),
        '#description' => t("Your occupation, e.g. 'Orthopedic Surgeon'."),
      );
      $element['aff_ref_type'] = array(
        '#type' => 'textfield',
        '#title' => t('Affiliation'),
        '#default_value' => nlmfield_process_xpath('//@rid', $xml),
        '#description' => t("Your organisation or institution (if applicable), e.g. 'Royal Free Hospital'.")
      );
      $element['address'] = array(
        '#type' => 'textfield',
        '#title' => t('Address'),
        '#default_value' => nlmfield_process_xpath('//addr-line', $xml),
        '#description' => t("Your organisation, institution's or residential address."),
      );
      $element['suffix'] = array(
        '#type' => 'textfield',
        '#title' => t('Name Style'),
        '#default_value' => nlmfield_process_xpath('//suffix', $xml),
      );
      $element['prefix'] = array(
        '#type' => 'textfield',
        '#title' => t('Name Prefix'),
        '#default_value' => nlmfield_process_xpath('//prefix', $xml),
      );
      $element['degrees'] = array(
        '#type' => 'textfield',
        '#title' => t('Degrees'),
        '#default_value' => nlmfield_process_xpath('//degrees', $xml),
      );
    break;
  }
  return $element;
}

/**
 * Implements hook_field_formatter_info().
 */
function nlmfield_field_formatter_info() {
  $formatters['nlmfield_xml'] = array(
    'label' => t('Full XML'),
    'field types' => array('nlmfield_contributor'),
    'multiple values' => CONTENT_HANDLE_CORE,
  );
  $presets = nlmfield_presets('nlmfield_contributor');

  foreach($presets as $i => $preset){
    $formatters[$i] = array(
      'label' => $preset['title'],
      'field types' => array('nlmfield_contributor'),
      'multiple values' => CONTENT_HANDLE_CORE,
    );
  }

  return $formatters;
}

/**
 * Theme function for text widget
 */
function theme_nlmfield_text($element){
  $output = '';
  $output .= theme($element['xml']['#type'] , $element['xml']);
  $output .= theme($element['surname']['#type'] , $element['surname']);
  $output .= theme($element['contributor_type']['#type'] , $element['contributor_type']);
  $output .= theme($element['string_name']['#type'] , $element['string_name']);
  $output .= theme($element['given_names']['#type'] , $element['given_names']);
  $output .= theme($element['mail']['#type'] , $element['mail']);
  $output .= theme($element['role']['#type'] , $element['role']);
  $output .= theme($element['aff_ref_type']['#type'] , $element['aff_ref_type']);
  $output .= theme($element['address']['#type'] , $element['address']);
  $output .= theme($element['suffix']['#type'] , $element['suffix']);
  $output .= theme($element['prefix']['#type'] , $element['prefix']);
  $output .= theme($element['degrees']['#type'] , $element['degrees']);

  $output .= theme($element['address']['#type'] , $element['address']);
  return $output;
}

/**
 * Theme function for xml widget
 */
function theme_nlmfield_xml($element){
  return theme($element['xml']['#type'] , $element['xml']);
}

/**
 * Theme function for all the formatters
 */
function theme_nlmfield_display_combination($element){
  if (!is_null($element['#item']['xml']))
    return nlmfield_process_preset('nlmfield_contributor', $element['#formatter'], $element['#item']['xml']);
}

function nlmfield_field_widget_text_to_xml($element, &$form_state) {
  $delta = $element['#delta'];
  $field_name = $element['#field_name'];
  $xml_placeholder = $form_state['values'][$field_name][$delta];
  $xml = nlmfield_process_xml($xml_placeholder);
  if ($xml_placeholder['xml'] != $xml) {
    form_set_value($element, $xml, $form_state);
  }
}

/**
 * Auxiliary function to generate the xml from an array of values
 */
function nlmfield_process_xml($xml_placeholder){
  $dom = new DOMDocument();
  $contrib_element = $dom->createElement('contrib');
  $contrib_type =  $xml_placeholder['contributor_type'] ? $xml_placeholder['contributor_type'] : 'author';
  $contrib_element->setAttribute('contrib-type', $contrib_type);
  $name_element = $dom->createElement('name');
  if ($xml_placeholder['surname']) {
   $surname_element = $name_element->appendChild($dom->createElement('surname', $xml_placeholder['surname']));
  }
  if ($xml_placeholder['given_names']) {
   $given_name_element = $name_element->appendChild($dom->createElement('given-names', $xml_placeholder['given_names']));
  }
  if ($xml_placeholder['string_name']) {
   $string_name_element = $name_element->appendChild($dom->createElement('string-name', $xml_placeholder['string_name']));
  }

  $name_element = $contrib_element->appendChild($name_element);

  if ($xml_placeholder['mail']) {
    $mail_element = $contrib_element->appendChild($dom->createElement('email', $xml_placeholder['mail']));
  }

  if ($xml_placeholder['prefix']) {
     $prefix_element = $contrib_element->appendChild($dom->createElement('prefix', $xml_placeholder['prefix']));
  }
  if ($xml_placeholder['suffix']) {
     $suffix_element = $contrib_element->appendChild($dom->createElement('suffix', $xml_placeholder['suffix']));
  }
  if ($xml_placeholder['degrees']) {
    $degree_element = $contrib_element->appendChild($dom->createElement('degrees', $xml_placeholder['degrees']));
  }

  if ($xml_placeholder['role']) {
    $role_element = $contrib_element->appendChild($dom->createElement('role', $xml_placeholder['role']));
  }

  if ($xml_placeholder['aff_ref_type']) {
    $xref_element = $dom->createElement('xref');
    $xref_element->setAttribute('ref-type', 'aff');
    $xref_element->setAttribute('rid', $xml_placeholder['aff_ref_type']);
    $xref_element = $contrib_element->appendChild($xref_element);
  }

  // @TODO just adding addr-line, though address contains lot of other subelements
  if ($xml_placeholder['address']) {
    $address_element = $dom->createElement("address");
    $address_line_element = $dom->createElement("addr-line", $xml_placeholder['address']);
    $adress_line_element = $address_element->appendChild($address_line_element);
    $address_element = $contrib_element->appendChild($address_element);
  }

  $xml = $dom->saveXML($contrib_element);
  return $xml;
}
